#!/bin/sh
#
#	Get gateway information
#	This script can return various information:
#	* FPGA Version : FPGA version flashed on the HW
#	* HAL Version  : current HAL version
#	* FW Version   : FW (or BSP) version - manufacturer dependent
#	* HW ID        : HW ID of the Gateway (HW revision)
#   * Serial       : Serial ID of the Gateway
#	* SKU          : Stock Keeping Unit if made available by the manufacturer
#   * LRR Version  : Current running LRR version
#   * OS Version   : Linux Kernel version running on the GW
#   * Chip ID MSB  : MSB ID of the GW used to get the AES Key from Semtech
#   * Chip ID LSB  : LSB ID of the GW used to get the AES Key from Semtech

. ${ROOTACT}/lrr/com/system_setting.sh
. ${ROOTACT}/lrr/com/system_api.sh

# script file(s) to be sourced
. ${ROOTACT}/lrr/com/_functions.sh


echo "Updating ${ROOTACT}/usr/etc/lrr/versions.ini file"

# Path to the versions.ini that will be created and populated with all the retrieved information
CUSTOM_INI_FILE=${ROOTACT}/usr/etc/lrr/versions.ini

# Custom script directory, to be used for development and testing only as it's overriding other script directories
CUSTOM_SCRIPT_DIR=$ROOTACT/usr/etc/lrr/cmd_shells
# Common script directory, to be used to store generic Gateway scripts
COMMON_SCRIPT_DIR=$ROOTACT/lrr/com/cmd_shells

TMP_VERSION_INI="/tmp/_version_ini$$"
TMP_LRR_INI="/tmp/_lrr_ini$$"
TMP_LRR_USR_INI="/tmp/_lrr_usr_ini$$"
TMP_ENV_VERSION_INI="/tmp/_version_env_ini$$"

MICMDOPTIONS="-e"
MICMD="$ROOTACT/lrr/moreini/moreini.x $MICMDOPTIONS"

nbboard=$(getIniConf "$ROOTACT/usr/etc/lrr/lgw.ini" "gen" "board")
if [ -z "$nbboard" ]; then
    nbboard=1
fi

execute_script() {

    _script=$1
    SystemGetFilePath "$CUSTOM_SCRIPT_DIR" "$_script"
    if [ ! -z "$sysfilepath" ]; then
        $sysfilepath
        return 1
    fi
    SystemGetFilePath "$COMMON_SCRIPT_DIR" "$_script"
    if [ ! -z "$sysfilepath" ]; then
        $sysfilepath
        return 1
    fi
	echo "unidentified"
}


update_version_ini_file() {

    #empty the versions.ini file
    true > $TMP_VERSION_INI
    echo "[versions]" >> $TMP_VERSION_INI
    echo "lrr_version=$lrrversion" >> $TMP_VERSION_INI
    echo "hal_version=$hal" >> $TMP_VERSION_INI
    echo "fpga_version=$fpga" >> $TMP_VERSION_INI
    echo "semtech_ref_design=$refdesign" >> $TMP_VERSION_INI
    echo "firmware_version=$fwver" >> $TMP_VERSION_INI
    echo "os_version=$osver" >> $TMP_VERSION_INI
    echo "hardware_version=$hwid" >> $TMP_VERSION_INI
    echo "serial_number=$serial" >> $TMP_VERSION_INI
    echo "sku=$sku" >> $TMP_VERSION_INI

    if [ "$refdesign" = "2.1" ]; then
        i=0
        while [ $i -lt $nbboard ]; do
            chip_lsb=$(execute_script get_chip_id_lsb.sh $i)
            chip_msb=$(execute_script get_chip_id_msb.sh $i)
            index=""
            if [ $nbboard -gt 1 ]; then
                index=" ($i)"
            fi

            # retrieve LSB and MSB
            echo "chip_id_lsb${index}=$chip_lsb" >> $TMP_VERSION_INI
            echo "chip_id_msb${index}=$chip_msb" >> $TMP_VERSION_INI

            # retrieve AES keys
            aeskey=$(getIniConf "$ROOTACT/usr/etc/lrr/custom.ini" "board:$i" "aeskey")
            if [ -z "$aeskey" ]; then
                aeskey=$(getIniConf "$ROOTACT/usr/etc/lrr/lrr.ini" "board:$i"i "aeskey")
            fi
            echo "aeskey${index}=$aeskey" >> $TMP_VERSION_INI
            i=$(($i + 1))
        done
    fi

    echo "cpu_count=$cpu_count" >> $TMP_VERSION_INI
    #for future info to be fetched
    #echo "xxx=$xxx" >> $CUSTOM_INI_FILE

    echo "[versions]" > $TMP_LRR_INI
    cat $ROOTACT/lrr/config/lrr.ini | grep '_version' >> $TMP_LRR_INI

    echo "[versions]" > $TMP_LRR_USR_INI
    cat $ROOTACT/usr/etc/lrr/lrr.ini | grep '_version' >> $TMP_LRR_USR_INI
    
    $MICMD -t "$CUSTOM_INI_FILE" -y -a "$TMP_LRR_INI"
    $MICMD -t "$CUSTOM_INI_FILE" -y -a "$TMP_LRR_USR_INI"
    $MICMD -t "$CUSTOM_INI_FILE" -y -a "$TMP_VERSION_INI"

    # Substitute env variable in versions.ini
    LRR_VERSION=$(cat ${ROOTACT}/lrr/Version)
    PARAMS=$(grep -v '^\s*#' $ROOTACT/usr/etc/lrr/_parameters.sh | tr '\n' ' ')
    ( echo "cat <<EOF" ; cat "$CUSTOM_INI_FILE"; echo EOF ) > $TMP_ENV_VERSION_INI && chmod +x $TMP_ENV_VERSION_INI
    sh -c "LRR_VERSION=$LRR_VERSION $PARAMS $TMP_ENV_VERSION_INI" > "$CUSTOM_INI_FILE"

}


# get_fpga_version.sh
#
# NOTE FOR INTEGRATORS : To be developped as every GW has a different device path for the Semtech LoRa device
# Will call the get_fpga_version.x binary, which will in turn return the current FPGA version
# For Ref Design V1.0, get_fpag_version.x will not return anything as there is no FPGA (0)
# For Ref Design V1.5, get_fpga_version.x requires no input parameter
# For Ref Design V2.1, get_fpga_version.x requires the SPI Device as parameter (ex: "get_fpga_version.x -d /dev/spidev0.0")
#
# Input parameter : none
# Output parameter : FPGA version (numerical version, no "v"), exemple : 28
#
# This script will return the FPGA version of the LoRa module

fpga=$(execute_script get_fpga_version.sh)

# get_hal_version.sh
#
# NOTE FOR INTEGRATORS : Generic function, this should not have to be developped
#
# Get HAL version will probe the LoRa chip to return the HAL version
# Input parameter : none
# Output parameter : HAL version (numerical version, example: 4.1.3)
#
# This script will call the get_hal_version.x binary, which will in turn return the HAL version, to be parsed if necessary

hal=$(execute_script get_hal_version.sh)

# get_fw_version.sh
#
# NOTE FOR INTEGRATORS : To be developped as every GW has a different way to report FW version
#
# Input parameter : none
# Output parameter : FW version of the gateway
#
# This script will return the FW version of the gateway

fwver=$(execute_script get_fw_version.sh)

# get_hwid.sh
#
# NOTE FOR INTEGRATORS : To be developped as every GW has a different way to report Hardware ID, if available
#
# Input parameter : none
# Output parameter : Hardware ID of the gateway
#
# This script will return the HW ID of the gateway

hwid=$(execute_script get_hwid.sh)

# get_os_version.sh
#
# NOTE FOR INTEGRATORS : This is a standard Linux command (uname -r) is called, this should be generic and not to be updated
#    unless the value in versions.ini is wrong
#
# Input parameter : none
# Output parameter : Linux version running on the GW
#
# This script will return the Linux version running on the gateway

osver=$(execute_script get_os_version.sh)


# get_serial.sh
#
# NOTE FOR INTEGRATORS : To be developped as every GW has a different way to report Serial number, if available
#
# Input parameter : none
# Output parameter : Serial number of the gateway
#
# This script will return the Serial number of the gateway

serial=$(execute_script get_serial.sh)

# get_sku.sh
#
# NOTE FOR INTEGRATORS : To be developped as every GW has a different way to report SKU, if available
#
# Input parameter : none
# Output parameter : Stock Keeping Unit of the gateway
#
# This script will return the SKU of the gateway

sku=$(execute_script get_sku.sh)

# get_lrr_version.sh
#
# NOTE FOR INTEGRATORS : Generic function, this should not have to be developped
#
# Input parameter : none
# Output parameter : LRR Version of the gateway
#
# This script will return the LRRID of the gateway

lrrversion=$(execute_script get_lrr_version.sh)


# get_chip_id_lsb.sh
#
# NOTE FOR INTEGRATORS : Generic function, this should not have to be developped, unless working on a Non Semtech Reference Design
#
# Input parameter : none
# Output parameter : LSB Chip ID of the gateway
#
# This script will return the LSB Chip ID of the gateway, needed to get the AES Key from Semtech in order to use the Geolocation feature

# call in update_version_ini_file


# get_chip_id_msb.sh
#
# NOTE FOR INTEGRATORS : Generic function, this should not have to be developped, unless working on a Non Semtech Reference Design
#
# Input parameter : none
# Output parameter : MSB Chip ID of the gateway
#
# This script will return the MSB Chip ID of the gateway, needed to get the AES Key from Semtech in order to use the Geolocation feature

# call in update_version_ini_file

# get_semtech_refdesign.sh
#
# NOTE FOR INTEGRATORS : To be developped for every GW
#
# Input parameter : none
# Output parameter : Semtech Reference Design of the gateway's LoRa module
#
# This script will return the Semtech Reference Design of the gateway's LoRa module

refdesign=$(execute_script get_semtech_refdesign.sh)

# get_cpu_info.sh
#
# Input parameter : none
# Output parameter : Cpu count in /proc/stat
#
# This script will return the count of cpu cores for /proc/stat processing

cpu_count=$(execute_script get_cpu_info.sh)

# PROTOTYPE to add a new information field in the versions.ini file
#
# get_xxx.sh
#
# NOTE FOR INTEGRATORS :
#
# Input parameters :
# Output parameters :
#
#
#
# xxx=$(execute_script get_xxx.sh)

#now that we got every piece of information, populate the versions.ini file
update_version_ini_file

echo "${ROOTACT}/usr/etc/lrr/versions.ini file updated"
