#!/bin/sh

##
## @file
## @author Actility
## @copyright Actility SA
## @brief Environment Variable Generator
##
## @details
## This script processes a template file containing directives used to create a configuration file
##

localdir=`dirname $0`

usage() {
    echo "$0: -f <variable definition> [ -r ]"
    echo "    -r reset existing values (default: preserve already defined values)"
}


RESET=n
INPUT=""
while [ "$1" != "" ]; do
    case $1 in
        -f)
            shift
            INPUT=$1
            shift
            ;;
        -r)
            RESET=y
            shift
            ;;
        *)
            usage
            exit 1
            ;;
    esac
done

if [ ! -f "$INPUT" ]; then
    usage
    exit 1
fi

#
# read the list of commands to test and create the configuration file
#

file=`basename $0`
d=`date`

cat <<EOF
#
# File automatically generated by $file
# on $d
#

EOF

error=0
msg=""

# definition of possible actions
DIRECTORY_CMD="dir"
DIRECTORY_WRITABLE_CMD="dirw"
LOCATION_CMD="location"
FILE_CMD="file"
FILE_WRITABLE_CMD="filew"
BIN_CMD="bin"
TEST_CMD="test"
CONTROL_CMD="control"
SET_CMD="set"

addError() {
    echo "# ---> $1"
    echo "echo \"$1\""
    msg="${msg}$1\n"
    error=1
}

defineEnvvar() {
    echo "$1=$2"
    eval "export $1=$2"
}

convertBool() {
    if [ "$1" = "0" ]; then
        echo "n"
    else
        echo "y"
    fi
}

getExecPath() {
    # name can contain environment variable then use eval to expand while testing
    _exe=`eval echo $1`
    if [ -x $_exe ]; then
        echo $1
        return
    fi
    # look into PATH
    _path=`which $1`
    if [ -x "${_path}" ]; then
        echo ${_path}
        return
    fi
    echo ""
}


processVariable() {

    envvar=$1
    action=$2
    constraint=$3
    remaining=$4

    if [ "$action" = "$BIN_CMD" ]; then
        res=""
        for val in $remaining; do
            res=`getExecPath $val`
            if [ "$res" != "" ]; then
                break
            fi
        done
        if [ "$res" = "" ]; then
            if [ "$constraint" = "mandatory" ]; then
                addError "${envvar}: cannot define variable with \"$action $remaining\""
            fi
        fi
        defineEnvvar $envvar "$res"
        return   
    fi

    # dir:  select the first readable directory
    # dirw: select the first writeable directory

    if [ "$action" = "$DIRECTORY_CMD" ] || [ "$action" = "$DIRECTORY_WRITABLE_CMD" ]; then
        res=""
        for val in $remaining; do
            # val can contain environment variable so use "eval"
            d=`eval echo $val`
            if [ -d "$d" ]; then
                res="${val}"
                if [ "$action" = "$DIRECTORY_WRITABLE_CMD" ]; then
                    # check if it is writeable
                    touch $val/evaltst 2> /dev/null && rm $val/evaltst
                    if [ $? -eq 0 ]; then
                        break
                    fi
                else
                    # only its presence is checked
                    break
                fi
            fi
        done
        if [ "$res" = "" ]; then
            if [ "$constraint" = "mandatory" ]; then
                addError "${envvar}: cannot define variable with \"$action $remaining\""
            fi
        fi
        defineEnvvar $envvar "$res"
        return
    fi

    # file:  select the first readable file
    # filew: select the first writeable file
    # important: file names can contain wildcard but it should return only one file

    if [ "$action" = "$FILE_CMD" ] || [ "$action" = "$FILE_WRITABLE_CMD" ] || [ "$action" = "$LOCATION_CMD" ]; then
        res=""
        for val in $remaining; do
            # val can contain environment variable so use "eval"
            count=`eval ls -C1 $val 2>/dev/null| wc -l`
            if [ $count -ne 1 ]; then
                if [ $count -gt 1 ]; then
                    addError "${envvar}: several files matching the rule"
                fi
                continue
            fi
            # 1 only
            f=`eval ls -C1 $val`
            if [ -f "$f" ]; then
                if [ "$action" = "$FILE_WRITABLE_CMD" ]; then
                    if [ -w "$f" ]; then
                        res=${f}
                        break
                    fi
                elif [ "$action" = "$LOCATION_CMD" ]; then
                    res=`dirname $f`
                else
                    res="${f}"
                    break
                fi
            fi
        done
        if [ "$res" = "" ]; then
            if [ "$constraint" = "mandatory" ]; then
                addError "${envvar}: cannot define variable with \"$action $remaining\""
            fi
        fi
        defineEnvvar $envvar "$res"
        return
    fi

    if [ "$action" = "$TEST_CMD" ]; then
        res=n
        eval "$remaining" > /dev/null 2>&1
        cr=$?
        if [ $cr = 0 ]; then
            res=y
        fi
        defineEnvvar $envvar "$res"
        return
    fi

    if [ "$action" = "$SET_CMD" ]; then
        b=n
        res=`eval "$remaining" 2>/dev/null`
        res="\"$res\""
        if [ "$res" = "" ]; then
            if [ "$constraint" = "mandatory" ]; then
                addError "${envvar}: cannot define variable with \"$action $remaining\""
            fi
        fi
        defineEnvvar $envvar "$res"
        return
    fi

    # control
    if [ "$action" = "$CONTROL_CMD" ]; then
        one=n
        for v in $remaining; do
            eval "val=\${$v}"
            if [ "$val" = "y" ]; then
                one=y
            fi
        done
        if [ "$one" = "n" ]; then
            addError "Error: no variable set to 'y' in ${remaining}"
        fi
        return
    fi
        
    echo "invalid action: $action"
    exit 1
}

clearline=1

while read -r oneline; do

    if [ "$clearline" = "1" ]; then
        line=""
        clearline=0
    fi

    # extract last character in a portable way
    c=`echo $oneline | sed -e 's/^.*\(.\)$/\1/'`
    if [ "$c" = "\\" ]; then
        # remove backslash and concatenate with previous lines
        oneline=`echo $oneline | sed -e 's/^\(.*\)\\$/\1/'`
        if [ "$line" = "" ]; then
            line="$oneline"
        else
            line="$line $oneline"
        fi
        continue
    else
        # concatenate with previous lines if any
        if [ "$line" = "" ]; then
            line="$oneline"
        else
            line="$line $oneline"
        fi
    fi

    clearline=1

    if [ "$line" = "" ]; then
        echo ""
        continue
    fi

    # get comments containing "name:" or "action:"

    c=`echo $line | cut -c1`
    if [ "$c" = "#" ]; then

        # it is a comment
        # detect comments containing either variable name or action to set it
        variable=`echo $line | grep "variable:"`
        if [ "$variable" != "" ]; then
            remaining=`echo $variable | awk -F':' '{ print $2 }'`
            # to remove spaces
            LAST_VARIABLE=`echo $remaining`
        fi
        action=`echo $line | grep "action:"`
        if [ "$action" != "" ]; then
            remaining=`echo $action | awk -F':' '{ print $2 }'`
            LAST_ACTION=`echo $remaining | awk '{ print $1 }'`
            LAST_CONSTRAINT=`echo $remaining | awk '{print $2 }'`
            LAST_PARAMETERS=`echo $remaining | awk '{print substr($0, index($0, $3))}'`
        fi
        echo "$line"
        continue

    else

        # is it a variable definition?

        s=`echo $line | grep "^.*="`
        if [ "$s" != "" ]; then
            # is it the last know variable
            current=`echo $s | awk -F'=' '{print $1}'`
            value=`echo $s | awk -F'=' '{print $2}'`
            if [ "$current" = "$LAST_VARIABLE" ]; then
                if [ "$value" = "" ] || [ "$RESET" = "y" ]; then
                    # process it
                    processVariable $LAST_VARIABLE $LAST_ACTION $LAST_CONSTRAINT "$LAST_PARAMETERS"
                else
                    defineEnvvar $current "$value"
                fi
            fi
        fi
    fi

done < $INPUT
    

if [ "$error" = 1 ]; then
    echo "" >&2
    echo "-------------------------------------" >&2
    echo "Error while building definition file from $INPUT" >&2
    printf "${msg}" >&2
    echo "-------------------------------------" >&2
    echo "" >&2
    exit 1
fi

exit 0
